Content.makeFrontInterface(600, 600);

/****************************************************************************
*                                                                           *
*                                                                           *
*                            METERING FUNCTION                              *
*                                                                           *
*                   Bill Evans and his team of ferrets                      *
*                                                                           *
****************************************************************************/

// This function is designed for single channels of audio, though it will
// automatically average multiple channels. Set your number of channels
// in the Tile's property editor. Even though we only access the peaks
// periodically (when our LAF function is called), we never miss a peak
// because HISE is iterating over every sample in the (forthcoming) audio 
// buffer. Here are the parameters in the Property inspector; because we
// are implementing our own rendering, some of these parameters will not
// affect our own rendering.
//
// FollowWorkspace: Ignore.
//
// Index: Ignore.
//
// UseSourceChannels: Whether you want to measure the signal level going into 
// the audio component you're referencing, or the output level.
//
// PaddingSize: Like CSS padding, this is empty space all around the inside of
// the Tile. Since we are using an LAF, this parameter is ignored.
//
// ShowMaxPeak: With native HISE rendering, this will display a separate 
// graphic element (a line) that represents the most recent peak. We do 
// this in our LAF, so turn this off (i.e., set it to zero).
//
// ChannelIndexes: An array of channels. Use [0] or [1] for left or right 
// channels. If your samples are mono (and you don't split your signal), 
// I'm guessing both channels are the same. For stereo, your array 
// would be [0,1]. See my comment below on single and multiple
// channels in my meter function.
//
// UpDecayTime and DownDecayTime: An envelope follower is applied, meaning
// that short peaks can get smoothed out if your attack is great than zero.
// As with any dynamics processing, this is particularly useful for
// samples that have (or don't have) relevant transients, like drums.
// I'm not certain, but I believe this operations affect the peaks that
// are reported to us, so setting these parameters will affect our LAF meter
// the same as if you used HISE's native rendering. I think.
//
// The peak values that we receive from HISE are floating point values.
// Any value of 1.0 or over is distorted. Zero is silence. Any value 
// between 0.0 and 1.0 represents a clean signal.

////////////////////////// LAF to Draw Our Meter ////////////////////////////

// ------------- Global Variables

// Create the LAF function for our meter.
const LAF_matrixPeakMeter = Content.createLocalLookAndFeel();

// We'll explain these variables when we use them.
var global_numberOfMeterUpdates = 0;
var	peakHoldHorizontalOffset = 0.0;

// ------------- Define Our Function

// Peak meter floating tile
LAF_matrixPeakMeter.registerFunction("drawMatrixPeakMeter", function(g, obj) {

	// Create variables to hold the dimensions of the peak meter component.
	var meterBoxStartX = obj.area[0];
	var meterBoxStartY = obj.area[1];
	var meterBoxWidth = obj.area[2];
	var meterBoxHeight = obj.area[3];
	
	// The peak array is a list of the peaks for each of audio channels we're using.
	var meterPeakArray = obj.peaks;
	
	// The length of this array is equal to the number of audio channels we're using. 
	var numberOfMeterChannels = obj.peaks.length;
	
	// These variables are explained when they are used in the function.
	var peakHoldWidth = 3;
	var peakAverage =0;
	var meterCornerRounding = 2;
	var meterBarWidth = meterBoxWidth;
    var meterBarNoiseFloor = 2;
    var meterBarArea;
    var highestPeak = 0;
    var peakHoldPeriod = 20;
    
    // Here we define the variables for all our colours.
    var safeMeterBarColour = Colours.green;
    var idealMeterBarColour = Colours.yellow;
    var warningMeterBarColour = Colours.orange;
    var clippedMeterBarColour = Colours.red;
    var peakHoldColour = Colours.white;
    
    // These levels are specified in decibels. They represent the different
    // data points in our meter.
    var safeMeterColourAmplitude = -5;
    var idealMeterAmplitude = -3;
    var dangerMeterAmplitude = -1;
    var clipedMeterAmplitude = 0;

	// ------------- Step 1: Gathering Information About Our Signal

	// We need a sum of all the peaks so we can find their average. Try to guess how
	// they'll be used based on their names.
	var peakSum = 0;
	
	// Iterate through the audio channels we've assigned to the peak meter in the Properties inspector.
    for (peak in meterPeakArray) { 
    	
    	// Add each peak to our sum.
    	peakSum = peakSum+ peak; 
    	
    	// Find the highest peak on any of the channels. We'll use this variable later, but not
    	// between function calls. It represents the highest peak on this iteration of our function.
    	// Because we may be averaging more than one channel we need to iterate this comparison
    	// within this loop.
    	if (peak> highestPeak) { highestPeak = peak; }
    }
    
    // Find our average peak.
    peakAverage = peakSum / numberOfMeterChannels;

	// If our average peak is above 1.0, set it to 1.0. We want to communicate
	// to the user if the signal has distorted, and anything 1.0 or over is distorted.
	// In the digital domain, any amount of distortion is bad, and that's what we 
	// want to communicate in our meter to the user.
	if (peakAverage > 1.0) { peakAverage = 1.0; }

	// Our meter display is made up of two parts: a bar that represents a clean signal,
	// and a reserved space to indicate distoration. We'll put the reserved space at the
	// top/end of the meter, indicating that the distorted signal is higher than the clean 
	// signal.
	var cleanSignalWidthLimit = meterBoxWidth - peakHoldWidth;
	
	// ------------- Step 2: Drawing the Clean Signal
	
	// Now we can compute the length, in pixels, of our clean signal.
	meterBarWidth = (meterBoxWidth-peakHoldWidth) * peakAverage;

	// HISE can detect very quiet signals. This includes signals so quiet, that's probably
	// not useful for the user to know. So we will only draw a signal bar if our
	// signal is above a certain level.
	if (meterBarWidth > (meterBarNoiseFloor)) {
		
		// Here, we define the dimensions of our signal bar. Based on the variables we created
		// above, we have all the information we need. In HISE, we use an array to
		// specify rectanges. The variables, in order, represent the starting horizontal point,
		// the starting vertical point, the width, and the height.
		var meterBarArea = [meterBoxStartX, meterBoxStartY, meterBarWidth, meterBoxHeight];
	
		// We're going to create a colour gradient to visually represent the contium of
		// amplitude values in the user's signal. We'll use this to colour the signal bar.
		g.setGradientFill([	
			
			// This colour represents the range of quiet, safe signals. The other two
			// paramters specify that this colour begins at the low-end of the meter.
			safeMeterBarColour, meterBoxStartX, meterBoxHeight/2,
			
			// This colour represents a very high signal, alterting the user that the
			// signal may distort soon. The other two parameters specify that this colour
			// is at the high end of the meter.
			warningMeterBarColour, meterBoxWidth, meterBoxHeight/2,
			
			// This value indicates that we are creating a linear, not circular, gradient.
			false,
			
			// After we've defined the colours at either end of our gradient, we can 
			// insert colours in beteween. This colour represents the highest value
			// of signal that we want to tell the user is a quiet signal. This ensures
			// that there are no colour gradiations within this spectrum. Gradients should
			// always be avoided, if possible, because the values in between must be
			// processed cognitively to ascertain their meaning by the user.
			// Unlike the first two colours, we only use one variable to define where in 
			// our gradient spectrum this colour occurs; it's a number from 0.0 (start of the
			// gardient) to 1.0 (end of the gradient).
			safeMeterBarColour, Engine.getGainFactorForDecibels(safeMeterColourAmplitude),
			
			// This last colour indicates that the signal level is ideal, though that's
			// context-dependent.
			idealMeterBarColour, Engine.getGainFactorForDecibels(idealMeterAmplitude),
		]);
		
		// Now, we draw our signal bar. Our first parameter specifies the location and
		// dimensions. The second parameter specifies how much (if any, which would be zero)
		// curvature we have the box's corners.
		g.fillRoundedRectangle(meterBarArea, meterCornerRounding);
	}
	
	// ------------- Step 3: Drawing the Peak Hold Bar
	
	// This variable is used to count how many times our drawing function has executed. You'll
	// see how we use it later in the funciton.
	global_numberOfMeterUpdates++;

	// We only draw a new peak hold bar if one of two things happen. Either the current peak is higher
	// than the peak hold bar in our meter, or enough time has elapsed for us to draw a new one.	
	if ((meterBarWidth > peakHoldHorizontalOffset) || (global_numberOfMeterUpdates > peakHoldPeriod)) { 

		// Let's store our new peak value. We store this value as the number of pixels to right 
		// of the meter's original to draw the left-most part of our peak hold bar.
		peakHoldHorizontalOffset = meterBarWidth;
		
		// Since we're drawing a new peak hold bar, we'll reset our timer that counts calls
		// to this function. 
		global_numberOfMeterUpdates = 0;
	}

	// We will draw different peak hold bars depending whether we have a clean or distorted signal.
	if (highestPeak<1.0) {
		
		// We'll use this colour for clean signal peak holds.
		g.setColour(peakHoldColour);
		
		// Our peak hold bar has a width, so to be sure it starts in the right place, we don't want to
		// start drawing it before the meter starts.
		if (peakHoldHorizontalOffset> (peakHoldWidth + 1)) { 
		
		// Heres, we draw the peak hold bar. Our parameters are similar to the clean signal bar. Once
		// again, we need to account for the width of our peak hold bar.
		g.fillRoundedRectangle([peakHoldHorizontalOffset-peakHoldWidth,meterBoxStartY,peakHoldWidth,meterBoxHeight], meterCornerRounding); 
		}
	
	// In this care, our signal is distorted. For this special case, we draw our signal bar immediately 
	// above/right of the highest position for the clean signal (and a peak hold of it). 
	} else {
			
			// Red is bad.
			g.setColour(Colours.red);
			
			// We'll draw our peak hold bar here. You can see that we're starting the bar on the first pixel
			// above/right of the highest clean signal bar or clean signal peak hold bar.
			g.fillRoundedRectangle([meterBoxWidth-peakHoldWidth,meterBoxStartY,peakHoldWidth,meterBoxHeight], meterCornerRounding); 			
	}
});function onNoteOn()
{
	
}
 function onNoteOff()
{
	
}
 function onController()
{
	
}
 function onTimer()
{
	
}
 function onControl(number, value)
{
	
}
 